#include "bldc.h"
#include "ctrl_module.h"

//	 BIT 
static u8_t GetBit(u8_t nIdx, u8_t cMask, u8_t cShift)
{
	u8_t cRetVal = 0;

	/* MISRA C Dir 4.14 :  ȿ  Ȯ */
	if(nIdx > NUM_INDEX)
	{
#if DEBUG_COVER
		cRetVal = ERR_INDEX;
		xil_printf("Ctrl_Module: Values greater than register range 13\r\n");
#endif
	}
	else
	{
		cRetVal = (u8_t)BLDC_mReadReg(BASE_ADDR, BLDC_S00_AXI_SLV_REG0_OFFSET + (nIdx * 4u));	//	Ͱ а
		cRetVal = (cRetVal & cMask) >> cShift;													//	0 Ȥ 1 
	}

	return cRetVal;
}

//	 BIT ϱ
static void SetBit(u8_t nIdx, u8_t cMask, u8_t cShift)
{
	u32_t cVal = 0, cVal2 = 0, cVal3 = 0x01;

	/* MISRA C Dir 4.14 :  ȿ  Ȯ */
	if(nIdx <= NUM_INDEX)
	{
		cVal = (u32_t)BLDC_mReadReg(BASE_ADDR, BLDC_S00_AXI_SLV_REG0_OFFSET + (nIdx * 4u));		//	Ͱ а

		if(cMask == 0x01u)
		{
			cVal2 = (u32_t)(cVal3 << cShift);		//	1 
			cVal |= cVal2;
		}
		else
		{
			cVal2 = (u32_t)(cVal3 << cShift);		//	0 
			cVal2 = ~cVal2;
			cVal &= cVal2;
		}

		BLDC_mWriteReg(BASE_ADDR, (BLDC_S00_AXI_SLV_REG0_OFFSET + (nIdx * 4u)), (u32)cVal);
	}
}

//	 byte 
static u32_t GetByte(u8_t nIdx)
{
	u32_t cRetVal = 0;

	if(nIdx <= NUM_INDEX)
	{
		cRetVal = BLDC_mReadReg(BASE_ADDR, BLDC_S00_AXI_SLV_REG0_OFFSET + (nIdx * 4u));
	}
	else
	{
#if DEBUG_COVER
		cRetVal = ERR_INDEX;
#endif
	}

	return cRetVal;
}

//	 Byte ϱ
static void SetByte(u8_t nIdx, u8_t cData)
{
	BLDC_mWriteReg(BASE_ADDR, (BLDC_S00_AXI_SLV_REG0_OFFSET + (nIdx * 4u)), (u32)cData);
}

//	   Ȯ
static void CheckOverCurrent(void)
{
	if(g_cOverCurrFlag == 0u)		//	 ˶ ߻ϸ Ͱ  , Ʈ /ϰ  g_cOverCurrFlag = 0 ϰ  
	{
		g_cOverCurrFlag = (u8_t)GetBit(REG_IDX_4, REG_MASK_OVER_CURR, SHIFT_OVER_CURR);
	}
	else
	{
		if((g_cOverCurrFlag	== 1u) && (g_cMastRcvCmd != 0x00u))		//	 ߻  Ʈ °  ƴ 츸 (1ȸ Կ)
		{
			SetBit(REG_IDX_7, 0x01, SHIFT_MAST_OVER_CURR);			//	01.   Ŭ, Register8, High, bit3

			g_cMastRcvCmd = 0x00u;									//	02. Ʈ 
			SetMotor(g_cMastRcvCmd);								//	Motor Up/Down/Stop  

			SetBit(REG_IDX_7, 0x00, SHIFT_MAST_OVER_CURR);			//	03.   Ŭ, Register8, Low, bit3
		}
	}
}

//	ڴ  ġ Ȯ
static void CheckEncoder(void)
{
	g_cEncoderErrFlag = (u8_t)GetBit(REG_IDX_4, REG_MASK_ENCODER_ERR, SHIFT_ENCODER_ERR);

	g_cEncoderHWResetFlag = (u8_t)GetBit(REG_IDX_4, REG_MASK_ENCODER_HW_RESET, SHIFT_ENCODER_HW_RESET);		//	̰ H/W  Է ˻

	if(g_cEncoderHWResetFlag == 1u)
	{
		SetBit(REG_IDX_7, 0x01, SHIFT_ENCODER_HW_RESET_CLEAR);			//	ڴ H/W Reset Clear Set, Register7

		g_cEncoderHWResetFlag = 0;
		g_nEncoderMT = 0;
		g_nMastCurrHeight = 0;
		SaveEncoderMT((u32)g_nEncoderMT);	//	ڴ Ƽϰ 

		SetBit(REG_IDX_7, 0x00, SHIFT_ENCODER_HW_RESET_CLEAR);			//	ڴ H/W Reset Clear Reset, Register7
	}
}

//	Bit 3 : Ͱ							g_cOverCurrFlag
//	Bit 2 : ̰ ǥÿ ڴ üũ			g_cEncoderErrFlag
//	Bit 1 : Ʈ  üũ					0
//	Bit 0 : ׳Ʈ-ӹ  		g_cBitDetailHBErr
//	BIT  Ȯ
static void CheckBitDetailInfo(void)
{
	u8_t cData = 0u;

	cData = 0x01u;
	if(g_cBitDetailHBErr == 1u)		//	׳Ʈ-ӹ  , Bit: 0
	{
		g_cBitDetail |= cData;
	}
	else
	{
		cData = ~cData;
		g_cBitDetail &= cData;
	}

	cData = 0x02;
	if((g_cBitDetailHBErr == 1u) || (g_cEncoderErrFlag == 1u) || (g_cOverCurrFlag == 1u))		//	Ʈ üũ( or ڴ or ) üũ, Bit: 1	24.09.25 BIT  
	{
		g_cBitDetail |= cData;
	}
	else
	{
		cData = ~cData;
		g_cBitDetail &= cData;
	}

	cData = 0x04;
	if(g_cEncoderErrFlag == 1u)			//	̰ ǥÿ ڴ üũ, Bit: 2
	{
		g_cBitDetail |= cData;
	}
	else
	{
		cData = ~cData;
		g_cBitDetail &= cData;
	}

	cData = 0x08;
	if(g_cOverCurrFlag == 1u)			//	Ͱ, Bit: 3
	{
		g_cBitDetail |= cData;
	}
	else
	{
		cData = ~cData;
		g_cBitDetail &= cData;
	}
}

//	̰ Ȯ
static void CheckCurrHeight(void)
{
	u32_t nRetVal = 0, nVal, nData;
	float32 fUpper, fLower, fData;

	nRetVal = BLDC_mReadReg(BASE_ADDR, BLDC_S00_AXI_SLV_REG6_OFFSET);	//	Encoder 
	g_nEncoderMT = nRetVal;
	/* Encoder 	*/
	nData = nRetVal & SINGLE_CYCLE_MAST;
	fLower = (float32)nData;											//	ȸ  (Ҽ )		0 ~ 1023: 0 ~ 32.0mm(0.0 ~ 31.97)
	fLower = fLower * (float32)SINGLE_BASE_SCALE;

	nData = nRetVal & MULTI_CYCLE_MAST;
	nData = nData >> 10u;
	fUpper = (float32)(nData);											//	ȸ  

	fData = fUpper * (float32)SINGLE_CYCLE_SCALE;
	fData = fData + fLower;
	nVal = (u32_t)fData;

	/* MISRA C Dir 4.14 : Ʈ  ִ밪 ʰ  ó */
	if(nVal > ANT_MAST_HEIGHT_MAX)
	{
		g_nMastCurrHeight = ANT_MAST_HEIGHT_MAX;
	}
	else
	{
		g_nMastCurrHeight = nVal;
	}

	/* ִ/ּҰ    ó */
	if(g_cMastRcvCmd == (u8_t)CMD_MOTOR_CTN_UP)							//	 
	{
		if(g_nMastCurrHeight >= ANT_MAST_HEIGHT_MAX)					//	ִ밪 
		{
			g_cMastRcvCmd = (u8_t)CMD_MOTOR_STOP;						//	Ʈ  Ŀǵ 
			SetMotor(g_cMastRcvCmd);									//	Ʈ Stop
		}
	}
	
	if(g_cMastRcvCmd == (u8_t)CMD_MOTOR_CTN_DOWN)						//	ϰ 
	{
		if(g_nMastCurrHeight <= ANT_MAST_HEIGHT_MIN)					//	ּҰ 
		{
			g_cMastRcvCmd = (u8_t)CMD_MOTOR_STOP;						//	Ʈ  Ŀǵ 
			SetMotor(g_cMastRcvCmd);									//	Ʈ Stop
		}
	}

	if(g_cMastRcvCmd == (u8_t)CMD_MOTOR_USER_INPUT)						//	 Է°   
	{
		if(g_cMastAction == (u8_t)DIR_MOTOR_UP)							//	 
		{
			if(g_nMastCurrHeight >= (g_nSetMastHeight - 30u))			//	Ʈ ӵ  offset
			{
				g_cMastRcvCmd = (u8_t)CMD_MOTOR_STOP;					//	Ʈ  Ŀǵ 
				SetMotor(g_cMastRcvCmd);								//	Ʈ Stop
			}
		}
		if(g_cMastAction == (u8_t)DIR_MOTOR_DOWN)						//	ϰ 
		{
			if(g_nMastCurrHeight <= (g_nSetMastHeight + 30u))
			{
				g_cMastRcvCmd = (u8_t)CMD_MOTOR_STOP;					//	Ʈ  Ŀǵ 
				SetMotor(g_cMastRcvCmd);								//	Ʈ Stop
			}
		}
	}

#if DEBUG_CTRL_MODULE
	xil_printf("g_nMastCurrHeight:%d, g_nSetMastHeight:%d, nRetVal:%d, fLower:%d, fUpper:%d, g_cMastRcvCmd:%d, g_cMastAction:%d\r\n",
	(u32_t)g_nMastCurrHeight, (u32_t)g_nSetMastHeight, (u32_t)nRetVal, (u32_t)fLower, (u32_t)fUpper, (u32_t)g_cMastRcvCmd, (u32_t)g_cMastAction);
#endif
}

//	Ʈ  (Stop, Up, Down: 0x00, 0x01, 0x02)
//	ControlItem(1) 0xC0:  Է, 0x80:  , 0x40:  ϰ, 0x00: 
//	/ϰ       ϴ  ߰ 24.04.07
//	  ϰ DevicePowerRequest     ߰ 24.05.10
void SetMotor(u8_t cCmd)
{
	if(cCmd == (u8_t)CMD_MOTOR_STOP)									//	01. 0x00 
	{
		if(g_cMastAction == DIR_MOTOR_UP)								//	1-1    
		{
			g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_UP;				//	 
		}
		else															//	1-2   ϰ 
		{
			g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_DOWN;				//	ϰ 
		}
		g_nEcdeepromCnt = 0;											//	  ī ʱȭ
		g_cEncoderSaveFlag = 1;											//	ڴ Ƽϰ eeprom  
	}
	if((cCmd == (u8_t)CMD_MOTOR_CTN_UP) && (g_cCmdPowerOff == 0u))		//	02. 0x80 , DevicePowerRequest Ȯ
	{
		if(g_cMastAction == (u8_t)DIR_MOTOR_DOWN)						//	2-1   ϰ 
		{
			g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_DOWN;				//	ϰ 
			g_nEcdeepromCnt = 0;										//	  ī ʱȭ
			g_cEncoderSaveFlag = 1;										//	ڴ Ƽϰ eeprom  

			g_cMastRcvCmd = 0x00u;		//	׳Ʈ  
		}
		else															//	2-2  
		{
			g_cMastAction = (u8_t)DIR_MOTOR_UP;							//	
		}
	}
	if((cCmd == (u8_t)CMD_MOTOR_CTN_DOWN) && (g_cCmdPowerOff == 0u))	//	03. 0x40 ϰ, DevicePowerRequest Ȯ
	{
		if(g_cMastAction == (u8_t)DIR_MOTOR_UP)							//	3-1    
		{
			g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_UP;				//	 
			g_nEcdeepromCnt = 0;										//	  ī ʱȭ
			g_cEncoderSaveFlag = 1;										//	ڴ Ƽϰ eeprom  

			g_cMastRcvCmd = 0x00u;		//	׳Ʈ  
		}
		else															//	3-2  
		{
			g_cMastAction = (u8_t)DIR_MOTOR_DOWN;						//	ϰ
		}
	}
	if((cCmd == (u8_t)CMD_MOTOR_USER_INPUT) && (g_cCmdPowerOff == 0u))	//	04. 0xC0  Է, DevicePowerRequest Ȯ
	{
		if(g_nMastCurrHeight < g_nSetMastHeight)						//	4-1  ̰ ǥ ̺  
		{
			if(g_cMastAction == (u8_t)DIR_MOTOR_DOWN)					//	4-1-1   ϰ 
			{
				g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_DOWN;			//	ϰ 
				g_nEcdeepromCnt = 0;									//	  ī ʱȭ
				g_cEncoderSaveFlag = 1;									//	ڴ Ƽϰ eeprom  

				g_cMastRcvCmd = 0x00u;		//	׳Ʈ  
			}
			else														//	4-1-2  
			{
				g_cMastAction = (u8_t)DIR_MOTOR_UP;						//	
			}
		}
		else															//	4-2  ̰ ǥ ̺ ų  ϰ
		{
			if(g_cMastAction == (u8_t)DIR_MOTOR_UP)						//	4-2-1    
			{
				g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_UP;			//	 
				g_nEcdeepromCnt = 0;									//	  ī ʱȭ
				g_cEncoderSaveFlag = 1;									//	ڴ Ƽϰ eeprom  

				g_cMastRcvCmd = 0x00u;		//	׳Ʈ  
			}
			else														//	4-2-2  
			{
				g_cMastAction = (u8_t)DIR_MOTOR_DOWN;					//	ϰ
			}
		}
	}

	if((g_cMastAction == (u8_t)DIR_MOTOR_UP) || (g_cMastAction == (u8_t)DIR_MOTOR_STOP_FOR_UP)
	|| (g_cMastAction == (u8_t)DIR_MOTOR_DOWN) || (g_cMastAction == (u8_t)DIR_MOTOR_STOP_FOR_DOWN))		//	ȿ ɸ 
	{
		BLDC_mWriteReg(BASE_ADDR, BLDC_S00_AXI_SLV_REG2_OFFSET, (u32)g_cMastAction);	//	Ʈ  
	}

#if DEBUG_CTRL_MODULE
	xil_printf("SetMotor: cCmd:%d, g_cMastAction:%d\r\n", (u32_t)cCmd, (u32_t)g_cMastAction);
#endif
}

//	LED  (˶, Ʈ )
static void SetLEDControl(void)
{
	u32_t cData = 0u, cMotor = 0u;

	cData = GetByte(REG_IDX_7);					//	LED  Ͱ 

	if((g_cBitDetail & BIT_DETAIL_STATUS_OVER_CURR) == BIT_DETAIL_STATUS_OVER_CURR)		//	 ˶, 250mSec ֱ  500mSec   ó
	{
		switch (g_cMotorLedFlickerCnt)
		{
		case 0: cMotor = LED_MOTOR_ON;		break;
		case 1: cMotor = LED_MOTOR_OFF;		break;
		default: cMotor = LED_MOTOR_OFF;	break;
		}

		if(g_cMotorLedFlickerCnt < 1u)
		{
			g_cMotorLedFlickerCnt++;
		}
		else
		{
			g_cMotorLedFlickerCnt = 0u;
		}
	}
	else																				//	 
	{
		if((g_cMastAction == DIR_MOTOR_UP) || (g_cMastAction == DIR_MOTOR_DOWN))
		{
			cMotor = LED_MOTOR_ON;
		}
		else
		{
			cMotor = LED_MOTOR_OFF;
		}
	}

	if(cMotor == 1u)
	{
		cData |= (1u << 0u);		//	0: ڸ
	}
	else
	{
		cData &= ~(1u << 0u);
	}

	SetByte(REG_IDX_7, (u8_t)cData);					//	ΰ    LED ó
}

//	Ʈ ڴ Ƽϰ 
static void SetEncoderMT(void)
{
	BLDC_mWriteReg(BASE_ADDR, (BLDC_S00_AXI_SLV_REG0_OFFSET + (5u * 4u)), (u32)g_nEncoderMT);

	SetBit(REG_IDX_5, 0x01, SHIFT_ENCODER_WRITE);		//	Encoder Register  Flag
}

//	  ʱ 
void InitCtrlModule(void)
{
	u32 cData;
	g_cMastRcvCmd = (u8_t)CMD_MOTOR_STOP;
	g_cMastAction = (u8_t)DIR_MOTOR_STOP_FOR_UP;
	g_cCMProcessIdx = 0;
	g_cCmdPreparePowerOff = 0;
	g_cCmdPowerOff = 0;
	g_cMotorLedFlickerCnt = 0;						//	 LED  ī

	g_cBitDetailHBErr = 0;							//	Heart Beat  Off 
	g_cOverCurrFlag = 0;							//	 ߻ 
	g_cEncoderHWResetFlag = 0;

	SetEncoderMT();									//	Ʈ ڴ Ƽϰ , eeprom  register 
}

//	   ó
void CtrlModuleProcess(void)
{
	if(g_nCnt2 > TIMER_MSEC250)
	{
		g_nCnt2 = 0;
		CheckCurrHeight();							//	̰ Ȯ
		switch(g_cCMProcessIdx)						//	л ó
		{
		case 0: SetLEDControl();		break;		//	LED  (˶, Ʈ )
		case 1: CheckOverCurrent();		break;		//	  Ȯ
		case 2: CheckEncoder();			break;		//	ڴ  ġ Ȯ
		case 3: CheckBitDetailInfo();	break;		//	BIT  Ȯ
		default: break;
		}

		g_cCMProcessIdx++;
		if(g_cCMProcessIdx > 3u)
		{
			g_cCMProcessIdx = 0;
		}
	}

	if(g_cEncoderSaveFlag == 0u)				//	eeprom   ƴϸ ī ʱȭ
	{
		g_nEcdeepromCnt = 0u;
	}
	else										//	eeprom   
	{
		if(g_nEcdeepromCnt > TIMER_SEC3)
		{
			g_cEncoderSaveFlag = 0u;			//	 
			g_nEcdeepromCnt = 0u;
			SaveEncoderMT((u32)g_nEncoderMT);	//	ڴ Ƽϰ 
		}
	}
}





















