#include "xparameters.h"
#include "netif/xadapter.h"
#include "platform.h"
#include "platform_config.h"
#include "lwipopts.h"
#include "xil_printf.h"
#include "sleep.h"
#include "lwip/priv/tcp_priv.h"
#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/tcp.h"
#include "xil_cache.h"

#include "define.h"
#include "main.h"
#include "ctrl_module.h"
#include "tcpipcomm.h"
#include "tcp_client.h"
#include "udp_client.h"
#include "eeprom.h"

#if LWIP_IPV6==1
#include "lwip/ip6_addr.h"
#include "lwip/ip6.h"
#else

#if LWIP_DHCP==1
#include "lwip/dhcp.h"
extern volatile u32_t dhcp_timoutcntr;
#endif
#define DEFAULT_IP_ADDRESS	"172.16.0.50"
#define DEFAULT_IP_MASK		"255.255.255.0"
#define DEFAULT_GW_ADDRESS	"172.16.0.1"
#endif /* LWIP_IPV6 */

extern volatile u32_t TcpFastTmrFlag;
extern volatile u32_t TcpSlowTmrFlag;

struct netif server_netif;

#if defined (__arm__) && !defined (ARMR5)
#if XPAR_GIGE_PCS_PMA_SGMII_CORE_PRESENT == 1 || \
		 XPAR_GIGE_PCS_PMA_1000BASEX_CORE_PRESENT == 1
u32_t ProgramSi5324(void);
u32_t ProgramSfpPhy(void);
#endif
#endif

#ifdef XPS_BOARD_ZCU102
#ifdef XPAR_XIICPS_0_DEVICE_ID
u32_t IicPhyReset(void);
#endif
#endif


#if LWIP_IPV6==1
static void print_ipv6(u32_t *msg, ip_addr_t *ip)
{
	print(msg);
	xil_printf(" %s\n\r", inet6_ntoa(*ip));
}
#else
static void print_ip(const char8 *msg, const ip_addr_t *ip)
{
	print(msg);
	xil_printf("%d.%d.%d.%d\r\n", (u32_t)ip4_addr1(ip), (u32_t)ip4_addr2(ip),
			(u32_t)ip4_addr3(ip), (u32_t)ip4_addr4(ip));
}

static void print_ip_settings(const ip_addr_t *ip, const ip_addr_t *mask, const ip_addr_t *gw)
{
	print_ip("Board IP:       ", ip);
	print_ip("Netmask :       ", mask);
	print_ip("Gateway :       ", gw);
}

static void assign_default_ip(const ip_addr_t *ip, const ip_addr_t *mask, const ip_addr_t *gw)
{
	u32_t err;

	xil_printf("Configuring default IP %s \r\n", DEFAULT_IP_ADDRESS);

	err = (u32_t)inet_aton(DEFAULT_IP_ADDRESS, ip);
	if (err == 0u)
	{
#if DEBUG_COVER
		xil_printf("Invalid default IP address: %d\r\n", (u32_t)err);
#endif
	}

	err = (u32_t)inet_aton(DEFAULT_IP_MASK, mask);
	if (err == 0u)
	{
#if DEBUG_COVER
		xil_printf("Invalid default IP MASK: %d\r\n", (u32_t)err);
#endif
	}

	err = (u32_t)inet_aton(DEFAULT_GW_ADDRESS, gw);
	if (err == 0u)
	{
#if DEBUG_COVER
		xil_printf("Invalid default gateway address: %d\r\n", (u32_t)err);
#endif
	}
}
#endif /* LWIP_IPV6 */


static void MakeTiming(void)
{

	if(++g_nCnt1 > TIMER_BASE)
	{
		g_nCnt1 = 0;
		g_nCnt2++;							//	ctrl_module.c	CtrlModuleProcess()
		g_nEcdeepromCnt++;					//	ctrl_module.c	SetMotor()
		g_nTcpCliConCnt++;					//	tcp_client.c	ConnectToTcpServer()
		g_nReportCnt++;						//	udp_client.c	UdpCommPeriod()
		
		if(g_stTcpClientMy.cConnected == TRUE)	//	client ӹó S/W(Server)  ¿ ī
		{
			g_nBitDetailHBCnt++;				//	udp_client.c	UdpCommProcess()
			g_nTcpDisconnectCnt++;				//	udp_client.c	
		}
	}
}

static void TcpipProcChk(struct netif* nif)
{
	if (TcpFastTmrFlag != 0u) {
		tcp_fasttmr();
		TcpFastTmrFlag = 0u;
	}
	if (TcpSlowTmrFlag != 0u) {
		tcp_slowtmr();
		TcpSlowTmrFlag = 0u;
	}
	(void)xemacif_input(nif);
}

u32_t main(void)
{
	struct netif *netif;
	u8_t mac_ethernet_address[] = { 0x48, 0x21, 0x0B, 0x5A, 0x37, 0xA7 };			/* the mac address of the board. this should be unique per board */
	u16_t nData = 0;
	u32_t nData32 = 0;

	netif = &server_netif;
	
	static ip_addr_t ipaddr, netmask, gw;

#if defined (__arm__) && !defined (ARMR5)
#if XPAR_GIGE_PCS_PMA_SGMII_CORE_PRESENT == 1 || \
		XPAR_GIGE_PCS_PMA_1000BASEX_CORE_PRESENT == 1
	ProgramSi5324();
	ProgramSfpPhy();
#endif
#endif

	/* Define this board specific macro in order perform PHY reset
	 * on ZCU102
	 */
#ifdef XPS_BOARD_ZCU102
	IicPhyReset();
#endif

	init_platform();

	xil_printf("\r\n\r\n");
	xil_printf("=====================================================\n\r");
	xil_printf("-----\tAntMastPh2 Control Module V%d.%d.%d.%d\t-----\n\r", (u32_t)VER_VERSION, (u32_t)VER_UPDATE, (u32_t)VER_TEST, (u32_t)VER_COMPILE);

#if TEST_LOG_ON
	xil_printf("-----\tCompiled by HKC\t[Debug]\t\t\t-----\n\r");
#else
	xil_printf("-----\tCompiled by HKC\t[Release]\t\t-----\n\r");
#endif
	xil_printf("=====================================================\n\r");

	g_szIpInfo[0] = MY_IP_ADDR1;
	g_szIpInfo[1] = MY_IP_ADDR2;
	g_szIpInfo[2] = MY_IP_ADDR3;
	g_szIpInfo[3] = MY_IP_ADDR4;

	IP4_ADDR(&ipaddr,	g_szIpInfo[0], g_szIpInfo[1], g_szIpInfo[2], g_szIpInfo[3]);
	IP4_ADDR(&netmask,	255, 255, 255, 0);									//	24.06.27 
	IP4_ADDR(&gw,		g_szIpInfo[0], g_szIpInfo[1], g_szIpInfo[2], 1);	//	24.06.27 

	lwip_init();		/* initialize lwIP */

	//if (!xemac_add(netif, NULL, NULL, NULL, mac_ethernet_address,
	/* Add network interface to the netif_list, and set it as default */
	if (!xemac_add((struct netif*)netif, (ip_addr_t*)&ipaddr, (ip_addr_t*)&netmask, (ip_addr_t*)&gw, (u8_t*)mac_ethernet_address, (u32_t)PLATFORM_EMAC_BASEADDR))
	{
#if DEBUG_COVER
		xil_printf("Error adding N/W interface\r\n");
#endif
	}
	else
	{
#if LWIP_IPV6==1
		netif->ip6_autoconfig_enabled = 1;
		netif_create_ip6_linklocal_address(netif, 1);
		netif_ip6_addr_set_state(netif, 0, IP6_ADDR_VALID);
		print_ipv6("\n\rlink local IPv6 address is:", &netif->ip6_addr[0]);
#endif /* LWIP_IPV6 */

		netif_set_default(netif);

		/* now enable interrupts */
		platform_enable_interrupts();

		/* specify that the network if is up */
		netif_set_up(netif);

#if (LWIP_IPV6==0)
#if (LWIP_DHCP==1)
	/* Create a new DHCP client for this interface.
	 * Note: you must call dhcp_fine_tmr() and dhcp_coarse_tmr() at
	 * the predefined regular intervals after starting the client.
	 */
	dhcp_start(netif);
	dhcp_timoutcntr = 24;
	while (((netif->ip_addr.addr) == 0) && (dhcp_timoutcntr > 0))
		xemacif_input(netif);

	if (dhcp_timoutcntr <= 0) {
		if ((netif->ip_addr.addr) == 0) {
			xil_printf("ERROR: DHCP request timed out\r\n");
			assign_default_ip(&(netif->ip_addr), &(netif->netmask), &(netif->gw));
		}
	}

	/* print IP address, netmask and gateway */
#else
		assign_default_ip((const ip_addr_t*)&(netif->ip_addr), (const ip_addr_t*)&(netif->netmask), (const ip_addr_t*)&(netif->gw));
#endif
		print_ip_settings((const ip_addr_t*)&(netif->ip_addr), (const ip_addr_t*)&(netif->netmask), (const ip_addr_t*)&(netif->gw));
#endif /* LWIP_IPV6 */

		g_cSn = 0;							//	Ŷ ۽ ī

		InitEeprom();						//	eeprom ʱȭ
		InitCtrlModule();					//	  ʱ 		//InitTcpServer();		//	TCP  ʱȭ

		InitTcpClient();					//	TCP Client ʱȭ
		InitUdpServer();					//	UDP Server ʱȭ		//InitUdpClient();		//	UDP Client ʱȭ

		InitCompareMember();				//	ڿ  ʱȭ
		while (1)
		{
			if(g_cDeviceErase == 0u)		//	Device Erase    ,			ϸ ü ߻ 24.03.05
			{
				MakeTiming();

				CtrlModuleProcess();		//	   ó		//TcpServCommProcess();	//	  TCP Ŭ̾Ʈ   ۽ ó

				TcpClientCommProceess();	//	TCP Client   ۽ ó
				ConnectToTcpServer();		//	TCP  ݺ  
				UdpCommProcess();			//	UDP  ó

				TcpipProcChk(netif);

			}
		}
	}

	return 0;
}







